@@ -22,6 +22,11 @@ export TEST_DEV
 rm -f "$RESULTS"
 
 check_kernel_support
+check_kernel_support_reiserfs
+# anything expected by common.convert
+check_global_prereq getfacl
+check_global_prereq setfacl
+check_global_prereq md5sum
 
 run_one_test() {
 	local testdir
@@ -36,10 +41,12 @@ run_one_test() {
 		# Only support custom test scripts
 		./test.sh
 		if [ $? -ne 0 ]; then
-			_fail "test failed for case $testname"
 			if [[ $TEST_LOG =~ dump ]]; then
-				cat "$RESULTS"
+				# the logs can be large and may exceed the
+				# limits, use 4MB for now
+				tail -c 3900000 "$RESULTS"
 			fi
+			_fail "test failed for case $testname"
 		fi
 	else
 		_fail "custom test script not found"
